@@ -1,5 +1,5 @@
-use crate::token::{Source, Token, TokenType, TokenValue};
-use crate::types::{FloatType, IntType, Number};
+use crate::tokens::{Token, TokenType, TokenValue};
+use crate::types::{FloatType, IntType, Number, Source};
 use crate::{debuggable, error, success};
 use std::mem;
 use std::str;
@@ -86,9 +86,25 @@ impl<'a> Context<'a> {
     }
 }
 
+pub struct LexerConfig {
+    // if use origin string, lexer won't escape special chars and keep the quotes or string boundaries.
+    pub use_origin_string: bool,
+    // reserve comments or not
+    pub reserve_comments: bool
+}
+
+impl LexerConfig {
+    pub fn default() -> Self {
+        LexerConfig {
+            use_origin_string: false,
+            reserve_comments: false
+        }
+    }
+}
+
 pub struct Lexer {
     debug: bool,
-    use_origin_string: bool,
+    config: LexerConfig,
     tokens: Vec<Token>,
 }
 
@@ -111,14 +127,13 @@ impl<'a> Lexer {
     pub fn new() -> Self {
         Lexer {
             debug: false,
-            use_origin_string: false,
             tokens: Vec::<Token>::new(),
+            config: LexerConfig::default()
         }
     }
 
-    // if use origin string, lexer won't escape special chars and keep the quotes or string boundaries.
-    pub fn set_use_origin_string(&mut self, use_origin_string: bool) {
-        self.use_origin_string = use_origin_string;
+    pub fn set_config(&mut self, config: LexerConfig) {
+        self.config = config;
     }
 
     pub fn run(&mut self, input: &'a str) -> Result<Vec<Token>, LexError> {
@@ -172,19 +187,33 @@ impl<'a> Lexer {
         ctx.skip(2);
         let sep_count = self.try_read_long_string_boundary(ctx, b'[');
         if sep_count >= 0 {
-            self.skip_long_string(ctx, sep_count as usize, "comment")?;
+            let comment = self.read_long_string_impl(ctx, sep_count as usize, "comment")?;
+            if self.config.reserve_comments {
+                success!((TokenType::MComment, TokenValue::Str(comment)))
+            } else {
+                Ok(None)
+            }
         } else {
-            self.read_short_comment(ctx);
+            self.read_short_comment(ctx)
         }
-        Ok(None)
     }
 
-    fn read_short_comment(&mut self, ctx: &mut Context) {
+    fn read_short_comment(&mut self, ctx: &mut Context) -> LexResult {
+        let mut bytes: Vec<u8> = Vec::new();
         while let Some(c) = ctx.get() {
             if Lexer::is_line_break(c) {
                 break;
             }
-            ctx.next();
+            ctx.skip_into(1, &mut bytes);
+        }
+        if let Ok(comment) = str::from_utf8(&bytes) {
+            if self.config.reserve_comments {
+                success!((TokenType::SComment, TokenValue::Str(comment.to_string())))
+            } else {
+                Ok(None)
+            }
+        } else {
+            lex_error!(self, ctx, "invalid single line comment")
         }
     }
 
@@ -432,14 +461,14 @@ impl<'a> Lexer {
     fn read_short_string(&mut self, ctx: &mut Context) -> LexResult {
         let mut bytes: Vec<u8> = Vec::new();
         let start = ctx.get();
-        if self.use_origin_string {
+        if self.config.use_origin_string {
             bytes.push(start.unwrap());
         }
         ctx.next();
         let unfinished_error: &'static str = "unfinished string";
         while ctx.get() != start {
             match ctx.get() {
-                Some(b'\\') if self.use_origin_string => ctx.skip_into(2, &mut bytes),
+                Some(b'\\') if self.config.use_origin_string => ctx.skip_into(2, &mut bytes),
                 Some(b'\\') => self.try_read_esc(ctx, &mut bytes)?,
                 Some(c) => {
                     if Lexer::is_line_break(c) {
@@ -452,7 +481,7 @@ impl<'a> Lexer {
                 None => return lex_error!(self, ctx, unfinished_error),
             }
         }
-        if self.use_origin_string {
+        if self.config.use_origin_string {
             bytes.push(ctx.get().unwrap());
         }
         if let Ok(string) = String::from_utf8(bytes) {
@@ -483,17 +512,17 @@ impl<'a> Lexer {
         -1
     }
 
-    // skip long string, return long string source info
-    fn skip_long_string(
+    // read long string
+    fn read_long_string_impl(
         &mut self,
         ctx: &mut Context,
         sep_count: usize,
         sem: &str,
-    ) -> Result<Source, LexError> {
+    ) -> Result<String, LexError> {
         let line = ctx.line;
         let mut start = 0;
 
-        if self.use_origin_string {
+        if self.config.use_origin_string {
             start = ctx.current - 2 - sep_count;
         }
 
@@ -502,27 +531,23 @@ impl<'a> Lexer {
             self.read_line_break(ctx)?;
         }
 
-        if !self.use_origin_string {
+        if !self.config.use_origin_string {
             start = ctx.current;
         }
-
-        let col = ctx.col;
+        
         while let Some(c) = ctx.get() {
             match c {
                 b']' => {
                     if self.try_read_long_string_boundary(ctx, b']') == sep_count as i8 {
                         let length;
-                        if self.use_origin_string {
+                        if self.config.use_origin_string {
                             length = ctx.current - start;
                         } else {
                             length = ctx.current - 2 - sep_count - start;
                         }
-                        return Ok(Source {
-                            line,
-                            pos: start,
-                            length,
-                            col,
-                        });
+                        if let Some(slice) = ctx.buffer.get(start..(start + length)) {
+                            return Ok(slice.to_string());
+                        }
                     } else {
                         ctx.next();
                     }
@@ -543,22 +568,8 @@ impl<'a> Lexer {
     fn read_long_string(&mut self, ctx: &mut Context) -> LexResult {
         let sep_count = self.try_read_long_string_boundary(ctx, b'[');
         if sep_count >= 0 {
-            let Source {
-                pos,
-                length,
-                col: _,
-                line: _,
-            } = self.skip_long_string(ctx, sep_count as usize, "string")?;
-            if let Some(slice) = ctx.buffer.get(pos..(pos + length)) {
-                let string = String::from(slice);
-                return success!((TokenType::String, TokenValue::Str(string)));
-            } else {
-                return lex_error!(
-                    self,
-                    ctx,
-                    &format!("invalid string slice ({}, {})", pos, length)
-                );
-            }
+            let string = self.read_long_string_impl(ctx, sep_count as usize, "string")?;
+            return success!((TokenType::String, TokenValue::Str(string)));
         }
         unreachable!()
     }
