@@ -1,7 +1,11 @@
-pub mod token;
-pub mod macros;
-pub mod lexer;
-pub mod types;
-pub mod parser;
 pub mod ast;
-pub mod ast_walker;
\ No newline at end of file
+pub mod ast_walker;
+pub mod compiler;
+pub mod consts;
+pub mod lexer;
+pub mod macros;
+pub mod opcodes;
+pub mod parser;
+pub mod tokens;
+pub mod types;
+pub mod proto;
