@@ -1,7 +1,8 @@
 use crate::{debuggable, error};
 
 use crate::ast::*;
-use crate::token::{Token, TokenType, TokenValue};
+use crate::tokens::{Token, TokenType, TokenValue};
+use crate::types::Source;
 
 pub struct Parser {
     tokens: Vec<Token>,
@@ -59,70 +60,85 @@ impl Parser {
 
     // block -> { stat [';'] }
     fn block(&mut self) -> ParseResult<Block> {
-        let mut stats: Vec<Stat> = Vec::new();
+        let mut stats: Vec<StatInfo> = Vec::new();
+        let saved = self.current_source();
         while !self.is_block_end() {
-            match self.current_token_type() {
-                TokenType::Return => {
-                    stats.push(self.stat()?);
-                    break;
-                }
-                _ => stats.push(self.stat()?),
+            let (stat, should_break) = match self.current_token_type() {
+                TokenType::Return => (self.stat()?, true),
+                _ => (self.stat()?, false),
+            };
+            if let Some(stat) = stat {
+                let source = self.current_source() - saved;
+                stats.push(StatInfo { source, stat });
+            }
+            if should_break {
+                break;
             }
         }
         Ok(Block { stats })
     }
 
-    fn stat(&mut self) -> ParseResult<Stat> {
+    fn stat(&mut self) -> ParseResult<Option<Stat>> {
         let line = self.current_line();
-        match self.current_token_type() {
+        let stat = match self.current_token_type() {
             // stat -> ';' (empty stat)
             TokenType::Semi => {
                 self.next();
-                Ok(Stat::Empty)
+                return Ok(None);
+            }
+            // stat -> comment
+            TokenType::SComment | TokenType::MComment => {
+                let stat = Stat::CommentStat(CommentStat {
+                    is_single_line: self.current_token_type() == TokenType::SComment,
+                    comment: self.current_token().get_string(),
+                });
+                self.next();
+                stat
             }
             // stat -> if stat
-            TokenType::If => Ok(Stat::IfStat(self.ifstat()?)),
+            TokenType::If => Stat::IfStat(self.ifstat()?),
             // stat -> while stat
-            TokenType::While => Ok(Stat::WhileStat(self.whilestat()?)),
+            TokenType::While => Stat::WhileStat(self.whilestat()?),
             // stat -> DO block END
             TokenType::Do => {
                 self.next();
                 let block = self.block()?;
                 self.check_match(TokenType::End, TokenType::Do, line)?;
-                Ok(Stat::DoBlock(DoBlock { block }))
+                Stat::DoBlock(DoBlock { block })
             }
             // stat -> forstat
-            TokenType::For => Ok(Stat::ForStat(self.forstat()?)),
+            TokenType::For => Stat::ForStat(self.forstat()?),
             // stat -> repeatstat
-            TokenType::Repeat => Ok(Stat::RepeatStat(self.repeatstat()?)),
+            TokenType::Repeat => Stat::RepeatStat(self.repeatstat()?),
             // stat -> funcstat
-            TokenType::Function => Ok(Stat::FuncStat(self.funcstat()?)),
+            TokenType::Function => Stat::FuncStat(self.funcstat()?),
             // stat -> localstat
             TokenType::Local => {
-                self.next();
+                self.next_and_skip_comment();
                 if self.test(TokenType::Function) {
-                    Ok(Stat::FuncStat(self.localfunc()?))
+                    Stat::FuncStat(self.localfunc()?)
                 } else {
-                    Ok(Stat::LocalStat(self.localstat()?))
+                    Stat::LocalStat(self.localstat()?)
                 }
             }
             // stat -> label
             TokenType::DbColon => {
-                self.next();
-                Ok(Stat::LabelStat(self.labelstat()?))
+                self.next_and_skip_comment();
+                Stat::LabelStat(self.labelstat()?)
             }
             // stat -> retstat
             TokenType::Return => {
-                self.next();
-                Ok(Stat::RetStat(self.retstat()?))
+                self.next_and_skip_comment();
+                Stat::RetStat(self.retstat()?)
             }
             // stat -> breakstat
-            TokenType::Break => Ok(Stat::BreakStat(self.breakstat()?)),
+            TokenType::Break => Stat::BreakStat(self.breakstat()?),
             // stat -> gotostat
-            TokenType::Goto => Ok(Stat::GotoStat(self.gotostat()?)),
+            TokenType::Goto => Stat::GotoStat(self.gotostat()?),
             // stat -> func | assignment
-            _ => Ok(self.exprstat()?),
-        }
+            _ => self.exprstat()?,
+        };
+        Ok(Some(stat))
     }
 
     // ifstat -> IF cond THEN block {ELSEIF cond THEN block} [ELSE block] END
@@ -133,9 +149,9 @@ impl Parser {
         while self.current_token_type() == TokenType::ElseIf {
             cond_blocks.push(self.test_then_block()?);
         }
-        let mut else_block = Block::empty();
+        let mut else_block = None;
         if self.test_next(TokenType::Else) {
-            else_block = self.block()?;
+            else_block = Some(self.block()?);
         }
         self.check_match(TokenType::End, TokenType::If, line)?;
         Ok(IfStat {
@@ -146,7 +162,7 @@ impl Parser {
 
     //  [IF | ELSEIF] cond THEN block
     fn test_then_block(&mut self) -> ParseResult<CondBlock> {
-        self.next();
+        self.next_and_skip_comment();
         let cond = self.cond()?;
         self.check_next(TokenType::Then)?;
         let block = self.block()?;
@@ -156,7 +172,7 @@ impl Parser {
     // whilestat -> WHILE cond DO block END
     fn whilestat(&mut self) -> ParseResult<WhileStat> {
         let line = self.current_line();
-        self.next();
+        self.next_and_skip_comment();
         let cond = self.cond()?;
         self.check_next(TokenType::Do)?;
         let block = self.block()?;
@@ -171,7 +187,7 @@ impl Parser {
     // forstat -> FOR (fornum | forlist) END
     fn forstat(&mut self) -> ParseResult<ForStat> {
         let line = self.current_line();
-        self.next();
+        self.next_and_skip_comment();
         let var_name = self.check_name()?;
         let forstat = match self.current_token_type() {
             TokenType::Assign => self.forenum(&var_name),
@@ -189,9 +205,10 @@ impl Parser {
 
     // fornum -> NAME = exp1,exp1[,exp1] forbody
     fn forenum(&mut self, var_name: &str) -> ParseResult<ForStat> {
-        self.next();
+        self.next_and_skip_comment();
         let init = self.expr()?;
         self.check_next(TokenType::Comma)?;
+        self.skip_comment();
         let limit = self.expr()?;
         let mut step = None;
         if self.test_next(TokenType::Comma) {
@@ -216,6 +233,7 @@ impl Parser {
             vars.push(self.check_name()?);
         }
         self.check_next(TokenType::In)?;
+        self.skip_comment();
         let exprs = self.exprlist()?;
         self.check_next(TokenType::Do)?;
         let body = self.block()?;
@@ -234,7 +252,7 @@ impl Parser {
 
     // funcstat -> FUNCTION funcname body
     fn funcstat(&mut self) -> ParseResult<FuncStat> {
-        self.next();
+        self.next_and_skip_comment();
         let func_name = self.funcname()?;
         let body = self.funcbody()?;
         Ok(FuncStat {
@@ -262,6 +280,7 @@ impl Parser {
     fn funcbody(&mut self) -> ParseResult<FuncBody> {
         let line = self.current_line();
         self.check_next(TokenType::Lp)?;
+        self.skip_comment();
         let mut params: Vec<Param> = Vec::new();
         loop {
             if self.test(TokenType::Rp) {
@@ -270,7 +289,7 @@ impl Parser {
             match self.current_token_type() {
                 TokenType::Dots => {
                     params.push(Param::VarArg);
-                    self.next()
+                    self.next_and_skip_comment()
                 }
                 TokenType::Name => params.push(Param::Name(self.check_name()?)),
                 _ => syntax_error!(self, "<name> or '...' expected")?,
@@ -287,7 +306,7 @@ impl Parser {
 
     // funcstat -> local FUNCTION funcname body
     fn localfunc(&mut self) -> ParseResult<FuncStat> {
-        self.next();
+        self.next_and_skip_comment();
         let func_name = self.funcname()?;
         let body = self.funcbody()?;
         Ok(FuncStat {
@@ -317,6 +336,7 @@ impl Parser {
     fn labelstat(&mut self) -> ParseResult<LabelStat> {
         let label = self.check_name()?;
         self.check_next(TokenType::DbColon)?;
+        self.skip_comment();
         Ok(LabelStat { label })
     }
 
@@ -331,12 +351,12 @@ impl Parser {
     }
 
     fn breakstat(&mut self) -> ParseResult<BreakStat> {
-        self.next();
+        self.next_and_skip_comment();
         Ok(BreakStat {})
     }
 
     fn gotostat(&mut self) -> ParseResult<GotoStat> {
-        self.next();
+        self.next_and_skip_comment();
         let label = self.check_name()?;
         Ok(GotoStat { label })
     }
@@ -362,6 +382,7 @@ impl Parser {
             left.push(self.suffixedexpr()?.to_assignable())
         }
         self.check_next(TokenType::Assign)?;
+        self.skip_comment();
         let right = self.exprlist()?;
         Ok(AssignStat { left, right })
     }
@@ -394,7 +415,7 @@ impl Parser {
         let mut left;
         let unop = self.get_unop();
         if unop != UnOp::None {
-            self.next();
+            self.next_and_skip_comment();
             let expr = Box::new(self.subexpr(unop.priority())?);
             left = Expr::UnExpr(UnExpr { op: unop, expr });
         } else {
@@ -402,7 +423,7 @@ impl Parser {
         }
         let mut binop = self.get_binop();
         while binop != BinOp::None && binop.priority().left > limit {
-            self.next();
+            self.next_and_skip_comment();
             let right = self.subexpr(binop.priority().right)?;
             left = Expr::BinExpr(BinExpr {
                 left: Box::new(left),
@@ -427,12 +448,12 @@ impl Parser {
             TokenType::Dots => Expr::VarArg,
             TokenType::Lb => return Ok(Expr::Table(self.table()?)),
             TokenType::Function => {
-                self.next();
+                self.next_and_skip_comment();
                 return Ok(Expr::FuncBody(self.funcbody()?));
             }
             _ => return Ok(self.suffixedexpr()?),
         };
-        self.next();
+        self.next_and_skip_comment();
         Ok(expr)
     }
 
@@ -443,17 +464,17 @@ impl Parser {
         loop {
             match self.current_token_type() {
                 TokenType::Attr => {
-                    self.next();
+                    self.next_and_skip_comment();
                     suffixes.push(Suffix::Attr(self.check_name()?));
                 }
                 TokenType::Ls => {
                     let line = self.current_line();
-                    self.next();
+                    self.next_and_skip_comment();
                     suffixes.push(Suffix::Index(self.expr()?));
                     self.check_match(TokenType::Rs, TokenType::Ls, line)?;
                 }
                 TokenType::Colon => {
-                    self.next();
+                    self.next_and_skip_comment();
                     let name = self.check_name()?;
                     suffixes.push(Suffix::Method(name));
                 }
@@ -480,7 +501,7 @@ impl Parser {
             TokenType::Name => Expr::Name(self.check_name()?),
             TokenType::Lp => {
                 let line = self.current_line();
-                self.next();
+                self.next_and_skip_comment();
                 let expr = self.expr()?;
                 self.check_match(TokenType::Rp, TokenType::Lp, line)?;
                 Expr::ParenExpr(Box::new(expr))
@@ -500,6 +521,7 @@ impl Parser {
     fn table(&mut self) -> ParseResult<Table> {
         let line = self.current_line();
         self.check_next(TokenType::Lb)?;
+        self.skip_comment();
         let mut fields: Vec<Field> = Vec::new();
         loop {
             if self.test(TokenType::Rb) {
@@ -508,6 +530,9 @@ impl Parser {
             fields.push(self.field()?);
             if !self.test_next(TokenType::Comma) && !self.test_next(TokenType::Semi) {
                 break;
+            } else {
+                // TODO : reverse comment for table fields
+                self.skip_comment();
             }
         }
         self.check_match(TokenType::Rb, TokenType::Lb, line)?;
@@ -537,13 +562,14 @@ impl Parser {
             TokenType::Name => key = FieldKey::Name(self.check_name()?),
             TokenType::Ls => {
                 let line = self.current_line();
-                self.next();
+                self.next_and_skip_comment();
                 key = FieldKey::Expr(self.expr()?);
                 self.check_match(TokenType::Rs, TokenType::Ls, line)?;
             }
             _ => unreachable!(),
         };
         self.check_next(TokenType::Assign)?;
+        self.skip_comment();
         let value = self.expr()?;
         Ok(Field::RecFileld(RecField { key, value }))
     }
@@ -558,7 +584,7 @@ impl Parser {
         let func_args = match self.current_token_type() {
             TokenType::Lp => {
                 let line = self.current_line();
-                self.next();
+                self.next_and_skip_comment();
 
                 // empty arg list
                 if self.test_next(TokenType::Rp) {
@@ -572,7 +598,7 @@ impl Parser {
             TokenType::Lb => FuncArgs::Table(self.table()?),
             TokenType::String => {
                 let arg = FuncArgs::String(self.current_token().get_string());
-                self.next();
+                self.next_and_skip_comment();
                 arg
             }
             _ => return syntax_error!(self, "function arguments expected"),
@@ -589,7 +615,11 @@ impl Parser {
     }
 
     fn next_token(&self) -> &Token {
-        &self.tokens[self.current + 1]
+        let mut current = self.current + 1;
+        while self.tokens[current].is_comment() {
+            current += 1;
+        }
+        &self.tokens[current]
     }
 
     fn current_token_type(&self) -> TokenType {
@@ -597,6 +627,11 @@ impl Parser {
         token.t
     }
 
+    fn current_source(&self) -> Source {
+        let token = self.current_token();
+        token.source
+    }
+
     fn current_line(&self) -> usize {
         let token = self.current_token();
         token.source.line
@@ -607,10 +642,23 @@ impl Parser {
         token.t
     }
 
+    fn next_and_skip_comment(&mut self) {
+        self.current += 1;
+        self.skip_comment();
+    }
+
     fn next(&mut self) {
         self.current += 1;
     }
 
+    fn skip_comment(&mut self) -> usize {
+        let old = self.current;
+        while self.current_token().is_comment() {
+            self.current += 1;
+        }
+        old
+    }
+
     // if reach a block end
     fn is_block_end(&self) -> bool {
         let token = self.current_token();
@@ -625,6 +673,7 @@ impl Parser {
     }
 
     fn check_match(&mut self, end: TokenType, start: TokenType, line: usize) -> ParseResult<()> {
+        self.skip_comment();
         if self.current_token_type() != end {
             if line == self.current_line() {
                 error_expected!(self, end);
@@ -644,10 +693,12 @@ impl Parser {
     }
 
     fn test_next(&mut self, expected: TokenType) -> bool {
+        let origin = self.skip_comment();
         if self.test(expected) {
             self.next();
             true
         } else {
+            self.current = origin;
             false
         }
     }
@@ -661,12 +712,14 @@ impl Parser {
     }
 
     fn check_next(&mut self, expected: TokenType) -> ParseResult<()> {
+        self.skip_comment();
         self.check(expected)?;
         self.next();
         Ok(())
     }
 
     fn check_name(&mut self) -> ParseResult<String> {
+        self.skip_comment();
         self.check(TokenType::Name)?;
         let token = self.current_token();
         let name = match &token.value {
